//
// -- LICENSE: The MIT License(MIT)
//
// Copyright(c) 2021 DinS - https://dins.site
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
#pragma once
#include <Urho3D/Container/Vector.h>
#include <Urho3D/Core/Context.h>
#include <Urho3D/IO/Deserializer.h>
#include <Urho3D/Resource/Resource.h>
#include <Urho3D/Resource/Image.h>
#include <Urho3D/Audio/Sound.h>
#include <Urho3D/Graphics/Texture2D.h>
#include <Urho3D/Urho2D/Sprite2D.h>
using Urho3D::SharedPtr;
using Urho3D::Vector;
using Urho3D::Resource;
using Urho3D::Context;
using Urho3D::Deserializer;
using Urho3D::Image;
using Urho3D::Texture2D;
using Urho3D::Sprite2D;
using Urho3D::Sound;

//https://github.com/phoboslab/pl_mpeg
struct plm_t;       //前置声明pl_mpeg

//解析Mpg资源音轨和图像(streaming)
class MpgVideo : public Resource
{
    URHO3D_OBJECT(MpgVideo, Resource);
public:
    MpgVideo(Context* context);
    virtual ~MpgVideo();
    static void RegisterObject(Context* context);
    virtual bool BeginLoad(Deserializer& source);       //耗时的基础性工作，worker thread
    virtual bool EndLoad();       //格式化为Urho3D资源，main thread

    double GetFrameRate() {return _dFps;}
    SharedPtr<Sound> GetAudio() {return _pAudioStream;}
    SharedPtr<Sprite2D> GetNextFrame();
private:
    //基本信息
    int _nWidth;
    int _nHeight;
    double _dFps;
    int _nFreq;
    plm_t* _pMpg;
    //音频
    uint8_t* _pRawAudio;                                            //原始音频流
    unsigned _nRawAudioLen;                                     //原始音频流长度
    SharedPtr<Sound> _pAudioStream;                     //格式化后的音轨
};
/*
播放视频的大问题：解析时间长 + 占用内存多
假设一个mpg是1280*900，时长18秒，很正常的一个数值
全部解析需要用若干秒，完全解析后的容量是
    1280 * 900 * 3 * 30 *    18        = 1780MB
        图像大小        fps     时长
所以处理思路是streaming，即需要帧图像时当场解析：decode as you watch；播放完一帧后立刻丢弃：discard as you watch
前提是CPU足够强大，在一次Urho3D::Update时限内可以完成
另外一旦图像渲染出来后Render2D会持有一个SharePtr<Material>，会阻止内存释放，需要做额外调整才能真正实现streaming
*/
