//
// -- LICENSE: The MIT License(MIT)
//
// Copyright(c) 2021 DinS - https://dins.site
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
#pragma once
#include "MpgVideo.h"
#include <Urho3D/Urho2D/StaticSprite2D.h>
using Urho3D::String;
using Urho3D::Scene;
using Urho3D::StringHash;
using Urho3D::VariantMap;
using Urho3D::StaticSprite2D;


//视频播放器，播放结束后自动移除并释放资源(one shot)
//这里的框架性代码主要根据AnimatedSprite2D仿写
class VideoPlayer : public StaticSprite2D
{
    URHO3D_OBJECT(VideoPlayer, StaticSprite2D);
    
public:
    VideoPlayer(Context* context);
    virtual ~VideoPlayer();
    static void RegisterObject(Context* context);
    void SetMpg(MpgVideo* pMpgRes, float fVolume);
    
protected:
    virtual void OnSceneSet(Scene* scene);
    //virtual void UpdateSourceBatches();       //使用StaticSprite2D提供的实现
    void HandleScenePostUpdate(StringHash eventType, VariantMap& eventData);
    void UpdateFrame(float timeStep);       //业务逻辑：更新帧图像
    
private:
    SharedPtr<MpgVideo> _pMpg;
    float _fFrameDuration;                //当前帧额定持续的时间(in seconds)，always same across frames
    float _fElapsedTime;                //当前帧已经流失的时间(in seconds)
};
